/////////////////////////////////////////////////////////
//
// Set the positions on a surface mesh.
//
//
//
//
// The surface is represented by a quadrilateral mesh, as
// depicted below. The integer-valued array-coordinates 
// run from 0 to sDim-1 in one direction, and from 0 to tDim-1 
// in the other.
//
//  ti = tDim-1 ------------------------- . . -------  t = 1.0   y =  1.0
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              -------------------------     -------
//              .     .     .     .                 .     .        .
//             
//              .     .     .     .                 .     .        .
//              -------------------------     -------
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//  ti = 3      ------------------------- . . -------
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//  ti = 2      ------------------------- . . -------
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//  ti = 1      ------------------------- . . -------
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//              |     |     |     |     |     |     |
//  ti = 0      ------------------------- . . -------  t = 0.0   y = -1.0
//       
//       
//        si =  0     1     2     3       . .    sDim-1
//       
//        s  =  0.0                       . .     1.0
//       
//        x  = -1.0                       . .     1.0
//       
//
// Floating point parameters s and t run from 0.0 to 1.0.
//
// Each vertex of the mesh has coordinates (x,y,z), with x and y
// running from -1.0 to 1.0, defining a square in physical space.


/////////////////////////////////////////////////////////////////////////////
//                                                                           
// Includes

#include <stdio.h>
#include <math.h>

#include "grid.h"


/////////////////////////////////////////////////////////////////////////////
//                                                                            
// Global Variables

extern GridStruct Grid;






/////////////////////////////////////////////////////////////////////////////
//                                    W     W                     W    W      
//                W         W         WW   WW              W     W      W     
//                W         W         W W W W              W     W      W     
// W  W  WWW    W W   WWW  WWW   WW   W  W  W   WW    WWW  WWW   W      W     
// W  W  W  W  W WW  W  W   W   W  W  W     W  W  W  W     W  W  W      W     
// W  W  W  W  W  W  W  W   W   WWWW  W     W  WWWW   WW   W  W  W      W     
// W  W  W  W  W  W  W WW   W   W     W     W  W        W  W  W  W      W     
//  WWW  WWW    WWW   W W   WW   WWW  W     W   WWW  WWW   W  W  W      W     
//       W                                                        W    W      
//       W                                                                    
//
// Set the positions on a surface mesh.
// Set the texture coordinates according to surface height.

void 
updateMesh() 
  {
  int      si, ti;          // loop variables for parameters s, t
  double   s, t;            // parameters and increments
  double   x, y, z;         // coordinates of a vertex 
  int      index;           // sequential index into array 

  Grid.coord-> point.setNum(Grid.sDim*Grid.tDim);

  SbVec3f *vertexPositions = Grid.coord-> point.startEditing();

  Grid.mesh->verticesPerRow    = Grid.sDim;
  Grid.mesh->verticesPerColumn = Grid.tDim;

  //
  // Main loop of the simulation.
  //
  // Traverse the grid and update the positions of the
  // vertices on the mesh.

  for (ti = 0; ti < Grid.tDim; ti++)
    {
    t =  4.0*M_PI*ti/(Grid.tDim-1.0);

    for (si = 0; si < Grid.sDim; si++)
      {
      s =  si/(Grid.sDim-1.0);

      x = s*cos(t)-0.5*s*s*cos(2.0*t);
      y = -s * sin(t)-0.5*s*s*sin(2.0*t);
      z = 4.0/3.0*pow(s, 1.5)*cos(1.5*t);

      // Put the newly-computed z-coordinate into the vertex (x,y,z).

      index = ti*Grid.sDim + si;

      vertexPositions[index].setValue( x,  y,  z);
      }
    }

  Grid.coord->point.finishEditing();
  }

